/***************************************************************************/
/*
 * recv.c --
 *
 * TCP/IP PVM-bandwidth test receiver.
 *
 * $Id: recv.c,v 1.2 95/07/24 18:30:56 tfisher Exp Locker: tfisher $
 *
 * Copyright (c) 1995 by the University of Southern California.
 * Permission to use, copy, modify, and distribute this software and its
 * documentation in source and binary forms for non-commercial purposes
 * and without fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both the copyright notice and
 * this permission notice appear in supporting documentation, and that
 * any documentation, advertising materials, and other materials related
 * to such distribution and use acknowledge that the software was
 * developed by the University of Southern California, Information
 * Sciences Institute.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * All rights reserved.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
 * the suitability of this software for any purpose.  This software
 * is provided "as is" and without any express or implied warranties,
 * including, without limitation, the implied warranties of merchant-
 * ibility and fitness for a particular purpose.
 *
 * For more information: 
 * Tom Fisher (tfisher@isi.edu) or Hong Xu (xu@isi.edu)
 */
/***************************************************************************/

#include <stdio.h>
#include "pvm3.h"

#define MSG_BEGIN	3
#define MSG_DATA	4

/* Externals. */
extern	double	calc_mbps();


int
main(int argc, char *argv[])
{
	int i;
	int local;		/* local task id */
	int parent;		/* parent task id */
	int size;		/* amount of data per iteration */
	int nloops;		/* number of receive iterations */
	char *data;		/* data buffer */
	double mbps;		/* communication bandwidth */
	struct timeval start;	/* first time stamp */
	struct timeval finish;	/* second time stamp */
	struct timeval elapsed;	/* difference between */

	/* Check for and get command line args. */
	if (parse_args(argc, argv, NULL, &size, &nloops) < 0)
		exit (1);

	/* Enroll in pvm. */
	local = pvm_mytid();
	parent = pvm_parent();

	/* 
	 * Make sure we send directly back to the parent
	 * as opposed to having the data routed through 
	 * the pvmd's on both machines (or only one pvmd 
	 * if both tasks are on the same machine). Having 
	 * task-to-task communication be routed through 
	 * pvmd(s) is the default.
	 */
	pvm_setopt(PvmRoute, PvmRouteDirect);

	/* Initialize. */
	data = (char *)alloc(size);
	pvm_initsend(PvmDataRaw);

	/* Receive the startup packet. */
	pvm_recv(parent, MSG_BEGIN);

	/* Wait for data from parent. */
	gettimeofday(&start, 0);
	for (i=0; i < nloops; i++)
		pvm_recv(parent, MSG_DATA);
	gettimeofday(&finish, 0);

	/* Calculate bandwidth. */
	timersub3(&finish, &start, &elapsed);
	mbps = calc_mbps(size, nloops, &start, &finish);

	/* Pack results for parent. */
	pvm_pkdouble(&mbps, 1, 1);
	pvm_pklong(&elapsed.tv_sec, 1, 1);
	pvm_pklong(&elapsed.tv_usec, 1, 1);

	/* Send results to parent. */
	pvm_send(parent, MSG_DATA);

	/* Print information. */
	printf(">>> recv: size=%d, loops=%d, time=%d.%06d, mbps=%.3f\n",
	    size, nloops, (int)elapsed.tv_sec, (int)elapsed.tv_usec, mbps);

	/* Program finished. Exit PVM before stopping */
	pvm_exit();
	exit(0);
}
