/***************************************************************************/
/*
 * send.c --
 *
 * TCP/IP PVM-bandwidth test generator.
 *
 * $Id: send.c,v 1.2 95/07/24 18:30:57 tfisher Exp Locker: tfisher $
 *
 * Copyright (c) 1995 by the University of Southern California.
 * Permission to use, copy, modify, and distribute this software and its
 * documentation in source and binary forms for non-commercial purposes
 * and without fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both the copyright notice and
 * this permission notice appear in supporting documentation, and that
 * any documentation, advertising materials, and other materials related
 * to such distribution and use acknowledge that the software was
 * developed by the University of Southern California, Information
 * Sciences Institute.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * All rights reserved.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
 * the suitability of this software for any purpose.  This software
 * is provided "as is" and without any express or implied warranties,
 * including, without limitation, the implied warranties of merchant-
 * ibility and fitness for a particular purpose.
 *
 * For more information: 
 * Tom Fisher (tfisher@isi.edu) or Hong Xu (xu@isi.edu)
 */
/***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "pvm3.h"

#define MSG_BEGIN	3
#define MSG_DATA	4
#define RECEIVER	"/nfs/atom/tfisher/src/PVM-bandwidth/pvm-tcp/recv"


int
main(int argc, char *argv[])
{
	int i;
	int local;		/* local task id */
	int child;		/* child task id */
	int size;		/* amount of data per iteration */
	int nloops;		/* number of send iterations */
	char host[64];		/* destination host name */
	char *data;		/* data buffer */
	double mbps;		/* communication bandwidth */
	struct timeval elapsed;	/* duration of test */

	/* Check for and get command line args. */
	if (parse_args(argc, argv, host, &size, &nloops) < 0)
		exit (1);

	/* Enroll in pvm. */
	local = pvm_mytid();

	/* Spawn a task to receive our data. */
	strcpy(argv[1], "keg-a.isi.edu");
	if (pvm_spawn(RECEIVER, argv+1, 
	    PvmTaskHost, host, 1, &child) != 1) {
		printf("%s: Can't spawn the receiver task... aborting.\n",
		    argv[0]);
		pvm_exit();
		exit (1);
	}

	/* 
	 * Make sure we send directly to our child task 
	 * as opposed to having the data routed through 
	 * the pvmd's on both machines (or only one pvmd 
	 * if both tasks are on the same machine). Having 
	 * task-to-task communication be routed through 
	 * pvmd(s) is the default.
	 */
	pvm_setopt(PvmRoute, PvmRouteDirect);

	/* Initialize. */
	data = (char *)alloc(size);
	pvm_initsend(PvmDataRaw);
	pvm_pkbyte(data, size, 1);

	/* Start test. */
	pvm_send(child, MSG_BEGIN);
	for (i=0; i < nloops; i++)
		pvm_send(child, MSG_DATA);

	/* Finish test. */
	pvm_recv(child, MSG_DATA);

	/* Unpack results. */
	pvm_upkdouble(&mbps, 1, 1);
	pvm_upklong(&elapsed.tv_sec, 1, 1);
	pvm_upklong(&elapsed.tv_usec, 1, 1);

	/* Print information. */
	printf(">>> send: size=%d, loops=%d, time=%d.%06d, mbps=%.3f\n",
	    size, nloops, (int)elapsed.tv_sec, (int)elapsed.tv_usec, mbps);

	/* Program Finished exit PVM before stopping */
	pvm_exit();
	exit (0);
}
