/***************************************************************************/
/*
 * recv.c --
 *
 * Myrinet PVM-latency test receiver.
 *
 * $Id: recv.c,v 1.1 95/07/25 11:10:26 tfisher Exp Locker: tfisher $
 *
 * Copyright (c) 1995 by the University of Southern California.
 * Permission to use, copy, modify, and distribute this software and its
 * documentation in source and binary forms for non-commercial purposes
 * and without fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both the copyright notice and
 * this permission notice appear in supporting documentation, and that
 * any documentation, advertising materials, and other materials related
 * to such distribution and use acknowledge that the software was
 * developed by the University of Southern California, Information
 * Sciences Institute.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * All rights reserved.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
 * the suitability of this software for any purpose.  This software
 * is provided "as is" and without any express or implied warranties,
 * including, without limitation, the implied warranties of merchant-
 * ibility and fitness for a particular purpose.
 *
 * For more information: 
 * Tom Fisher (tfisher@isi.edu) or Hong Xu (xu@isi.edu)
 */
/***************************************************************************/

#include <stdio.h>
#include "pvm3.h"
#include "atp.h"


int
main(int argc, char *argv[])
{
	int i;
	int unit=0;		/* myrinet board number */
	int interface=1;	/* myrinet interface number */
	int local;		/* local task id */
	int parent;		/* parent task id */
	int size;		/* amount of data per iteration */
	int nloops;		/* number of receive iterations */
	int sessID;		/* myrinet connection ID */
	int hostID;		/* peer's host myrinet ID */
	char *data;		/* data buffer */
	char host[128];		/* destination hostname */

	/* Check for and get command line args. */
	if (parse_args(argc, argv, host, &size, &nloops) < 0)
		exit (1);

	/* Initialize. */
	data = (char *)alloc(size);
	hostID = AtomicGetIdFromName(host);
	sessID = atp_open(unit, interface, host);

	/* Enroll in pvm. */
	local = pvm_mytid();
	parent = pvm_parent();

	/* Echo data. */
	pvm_precv(sessID, 0, data, 4, PVM_BYTE, 0, 0, 0);
	for (i=0; i < nloops; i++) {
		pvm_precv(sessID, 0, data, size, PVM_BYTE, 0, 0, 0);
		pvm_psend(sessID, hostID, data, size, 0);

	}

	/* Print information. */
	atp_print_receive_stats(sessID);

	/* Program finished. Exit PVM before stopping */
	atp_close(sessID);
	pvm_exit();
	exit(0);
}
