/***************************************************************************/
/*
 * send.c --
 *
 * Myrinet PVM-latency test generator.
 *
 * $Id: send.c,v 1.1 95/07/25 11:10:27 tfisher Exp Locker: tfisher $
 *
 * Copyright (c) 1995 by the University of Southern California.
 * Permission to use, copy, modify, and distribute this software and its
 * documentation in source and binary forms for non-commercial purposes
 * and without fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both the copyright notice and
 * this permission notice appear in supporting documentation, and that
 * any documentation, advertising materials, and other materials related
 * to such distribution and use acknowledge that the software was
 * developed by the University of Southern California, Information
 * Sciences Institute.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * All rights reserved.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
 * the suitability of this software for any purpose.  This software
 * is provided "as is" and without any express or implied warranties,
 * including, without limitation, the implied warranties of merchant-
 * ibility and fitness for a particular purpose.
 *
 * For more information: 
 * Tom Fisher (tfisher@isi.edu) or Hong Xu (xu@isi.edu)
 */
/***************************************************************************/

#include <stdio.h>
#include <string.h>
#include "pvm3.h"
#include "atp.h"

#define RECEIVER	"/nfs/atom/tfisher/src/PVM-latency/pvm-atp/recv"


int
main(int argc, char *argv[])
{
	int i;
	int unit=0;		/* myrinet unit to use */
	int interface=1;	/* myrinet interface to use */
	int local;		/* local task id */
	int child;		/* child task id */
	int size;		/* amount of data per iteration */
	int nloops;		/* number of send iterations */
	int hostID;		/* unique host ID */
	int sessID;		/* unique session descripter */
	char host[128];		/* destination host */
	char *data;		/* data buffer */
	double usecs;		/* communication delay in microseconds */
	struct timeval start;	/* first time stamp */
	struct timeval finish;	/* second time stamp */
	struct timeval elapsed;	/* total time spent */

	/* Check for and get command line args. */
	if (parse_args(argc, argv, host, &size, &nloops) < 0)
		exit(1);

	/* Initialize. */
	data = (char *)alloc(size);
	hostID = AtomicGetIdFromName(host);
	sessID = atp_open(unit, interface, host);

	/* Enroll in pvm. */
	local = pvm_mytid();

	/* Spawn a task to receive our data. */
	strcpy(argv[1], "keg-a.isi.edu");
	if (pvm_spawn(RECEIVER, argv+1, 
	    PvmTaskHost, host, 1, &child) != 1) {
		printf("%s: Can't spawn the receiver task... aborting.\n",
		    argv[0]);
		pvm_exit();
		exit (1);
	}

	/* 
	 * Make sure we send directly to our child task 
	 * as opposed to having the data routed through 
	 * the pvmd's on both machines (or only one pvmd 
	 * if both tasks are on the same machine). Having 
	 * task-to-task communication be routed through 
	 * pvmd(s) is the default.
	 */
	pvm_setopt(PvmRoute, PvmRouteDirect);

	/* Start receiver's timer. */
	pvm_psend(sessID, hostID, data, 4, PVM_BYTE);

	/* Begin the test. */
	gettimeofday(&start, 0);
	for (i=0; i < nloops; i++) {
		pvm_psend(sessID, hostID, data, size, PVM_BYTE);
		pvm_precv(sessID, 0, data, size, PVM_BYTE, 0, 0, 0);

	}

	/* Finish the test. */
	gettimeofday(&finish, 0);
	timersub3(&finish, &start, &elapsed);

	/* Get average latency in micro-seconds. */
	usecs = (double)((elapsed.tv_sec * 1000000) + elapsed.tv_usec) /
	    (nloops * 2);

	/* Print information. */
        printf(">>> send: size=%d, loops=%d, time=%d.%06d, usecs=%.0f\n",
            size, nloops, (int)elapsed.tv_sec, (int)elapsed.tv_usec, usecs);
        atp_print_send_stats(sessID);

	/* Program finished exit PVM before stopping */
	atp_close(sessID);
	pvm_exit();
	exit(0);
}
