/***************************************************************************/
/*
 * util.c -- 
 *
 * Common utility routines.
 *
 * $Id: util.c,v 1.1 95/07/25 11:10:27 tfisher Exp Locker: tfisher $
 *
 * Copyright (c) 1995 by the University of Southern California.
 * Permission to use, copy, modify, and distribute this software and its
 * documentation in source and binary forms for non-commercial purposes
 * and without fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both the copyright notice and
 * this permission notice appear in supporting documentation, and that
 * any documentation, advertising materials, and other materials related
 * to such distribution and use acknowledge that the software was
 * developed by the University of Southern California, Information
 * Sciences Institute.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * All rights reserved.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
 * the suitability of this software for any purpose.  This software
 * is provided "as is" and without any express or implied warranties,
 * including, without limitation, the implied warranties of merchant-
 * ibility and fitness for a particular purpose.
 *
 * For more information: 
 * Tom Fisher (tfisher@isi.edu) or Hong Xu (xu@isi.edu)
 */
/***************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>

/* Routines. */
int	parse_args();
char	*alloc();
char	*basename();
void	timeradd();
void	timersub2();
void	timersub3();
double	calc_mbps();
double	calc_mbps_alt();

int
parse_args(
	int argc,
	char *argv[],
	char *host,
	int *size,
	int *nloops)
{
	if (argc != 4) {
		printf("Usage: %s host packetsize niterations\n", 
		    basename(argv[0]));
		return (-1);
	}
	if (host != NULL)
		strcpy(host, argv[1]);
	*size = atoi(argv[2]);
	*nloops = atoi(argv[3]);
	if (*nloops < 0 || *nloops > 1000000) {
		printf("%s: Bad 'niterations' arg.\n", argv[0]);
		printf("Usage: %s host packetsize niterations\n",
		    basename(argv[0]));
		return (-1);
	}
	return (0);
}

/*
 * alloc() --
 * Allocate a properly aligned buffer.
 */
char *
alloc(int size)
{
	int adj_size; 
	int buf_align;
	int buf_offset;
	char *buf;
	char *buf_base;

	buf_align = 8;
	buf_offset = 0;

	adj_size = size + buf_align + buf_offset;
	buf_base = (char *)malloc(adj_size);
	if (buf_base == 0) {
		printf("malloc() failed.\n");
		exit(1);
	}

	buf = (u_char *) (((long)buf_base + 
	    (long)buf_align - 1) & ~((long)buf_align - 1));
	buf = buf + buf_offset;
	return (buf);
}

char *
basename(s)
	char *s;
{
	register char *p;

	p = s + strlen (s) - 1;
	while (p >= s && *p != '/')
		--p;
	return (++p);
}

void
timeradd(
	struct timeval *t0,
	struct timeval *t2)
{
	struct timeval tmp;

	timerclear(&tmp);
	tmp.tv_sec = t0->tv_sec + t2->tv_sec;
	tmp.tv_usec = t0->tv_usec + t2->tv_usec;
	if (tmp.tv_usec >= 1000000) {
		tmp.tv_sec++;
		tmp.tv_usec -= 1000000;
	}
	/* Put result into t0. */
	t0->tv_sec = tmp.tv_sec;
	t0->tv_usec = tmp.tv_usec;
}

void
timersub2(
	struct timeval *t0,
	struct timeval *t2)
{
	struct timeval tmp;

	timerclear(&tmp);
	tmp.tv_sec = t0->tv_sec - t2->tv_sec;
	tmp.tv_usec = t0->tv_usec - t2->tv_usec;
	if (t0->tv_usec < t2->tv_usec) {
		tmp.tv_sec--;
		tmp.tv_usec += 1000000;
	}
	/* Put result into t0. */
	t0->tv_sec = tmp.tv_sec;
	t0->tv_usec = tmp.tv_usec;
}

void
timersub3(
	struct timeval *t0,
	struct timeval *t1,
	struct timeval *out)
{
	timerclear(out);
	out->tv_sec = t0->tv_sec - t1->tv_sec;
	out->tv_usec = t0->tv_usec - t1->tv_usec;
	if (t0->tv_usec < t1->tv_usec) {
		out->tv_sec--;
		out->tv_usec += 1000000;
	}
}

double
calc_mbps_alt(
	int size,
	int nloops,
	struct timeval *time)
{
	double mbps=0;

	mbps = (double)(size * nloops * 8) / 
	    ((time->tv_sec * 1000000) + time->tv_usec);
	return (mbps);
}

double
calc_mbps(
	int size,
	int nloops,
	struct timeval *start,
	struct timeval *finish)
{
	double delta;
	double mbps=0;

	delta = (double)finish->tv_sec - (double)start->tv_sec;
	delta += (double)finish->tv_usec/1e6 - (double)start->tv_usec/1e6;

	mbps = (((double)nloops)*size)*8/1e6/delta;
	return (mbps);
}
