/***************************************************************************/
/*
 * atp_api_util.c --
 *
 * $Id: atp_api_util.c,v 1.3 95/06/01 12:19:10 tfisher Exp Locker: tfisher $
 *
 * Copyright (c) 1995 by the University of Southern California.
 * Permission to use, copy, modify, and distribute this software and its
 * documentation in source and binary forms for non-commercial purposes
 * and without fee is hereby granted, provided that the above copyright
 * notice appear in all copies and that both the copyright notice and
 * this permission notice appear in supporting documentation, and that
 * any documentation, advertising materials, and other materials related
 * to such distribution and use acknowledge that the software was
 * developed by the University of Southern California, Information
 * Sciences Institute.  The name of the University may not be used to
 * endorse or promote products derived from this software without
 * specific prior written permission.
 * All rights reserved.
 *
 * THE UNIVERSITY OF SOUTHERN CALIFORNIA makes no representations about
 * the suitability of this software for any purpose.  This software
 * is provided "as is" and without any express or implied warranties,
 * including, without limitation, the implied warranties of merchant-
 * ibility and fitness for a particular purpose.
 *
 * For more information: 
 * Tom Fisher (tfisher@isi.edu) or Hong Xu (xu@isi.edu)
 */
/***************************************************************************/

#include <stdio.h>
#include <errno.h>
#include <varargs.h>
#include <sys/types.h>
#include <sys/time.h>

/* Exported routines. */
void	Print(char *, int);
void	Error(int, char *, int);
void	TimeGet();
void	TimeAdd();
void	TimeSub();
int	TimeCompare();
double	TimeUSeconds();


inline void
Print(fmt, va_alist)
	char *fmt;
	va_dcl
{
#ifdef DEBUG
	va_list ap;

	va_start(ap);
	vfprintf(stdout, fmt, ap);
	va_end(ap);
	fflush(stdout);
#endif
}

void
Error(type, fmt, va_alist)
	int type;
	char *fmt;
	va_dcl
{
	va_list ap;

#ifndef DEBUG
	if (type) 
#endif
	{
		va_start(ap);
		vfprintf(stderr, fmt, ap);
		va_end(ap);
		putc('\n', stderr);
		fflush(stderr);
	}
	if (type)
		exit(1);
}

inline void
TimeGet(
	struct timeval *tv)
{
	(void)gettimeofday(tv, 0);
}

void
TimeAdd(
	struct timeval *t0,
	struct timeval *t1)
{
	t0->tv_sec = t0->tv_sec + t1->tv_sec;
	t0->tv_usec = t0->tv_usec + t1->tv_usec;
	if (t0->tv_usec > 1000000) {
		t0->tv_sec++;
		t0->tv_usec -= 1000000;
	}
}

void
TimeSub(
	struct timeval *t0,
	struct timeval *t1)
{
	t0->tv_sec = t0->tv_sec - t1->tv_sec;
	t0->tv_usec = t0->tv_usec - t1->tv_usec;
	if (t0->tv_usec < 0) {
		t0->tv_sec--;
		t0->tv_usec += 1000000;
	}
}

int
TimeCompare(
	struct timeval *t0,
	struct timeval *t1)
{
	if (t0 == 0)
		return (0);
	if (timerisset(t0) == 0)
		return (0);
	if (t1 == 0)
		return (1);
	if (t0->tv_sec > t1->tv_sec)
		return (1);
	if (t0->tv_sec < t1->tv_sec)
		return (-1);
	if (t0->tv_usec > t1->tv_usec)
		return (1);
	if (t0->tv_usec < t1->tv_usec)
		return (-1);
	return (0);
}

double
TimeUSeconds(
	struct timeval *tv)
{
	double usecs;

	usecs = (double)(tv->tv_sec * 1000000) + tv->tv_usec;
	return (usecs);
}
